/*Converte Records from MemoPad to SuperMemo.*/

//`General Setting;
#define    MPDB        "MemoDB"
#define    MPCATF    "Voc_New"
#define    MPCATT   "Voc_Cvtd"
#define    SMDB        "Vocabulary"
#define    SMCAT      "Unfiled"

//`Global Variables;
string  SmDB;
int    MPCatIDf, MPCatIDt,
         SMCatID, SMCatRecID;
string  Field[6];
int    FieldLoc[6], FieldLen[6];
int    MaxMPDBCats=0x10,
          MaxFieldLen=0xff,
          MaxSMDBRecs=0x10000;
          
/*Functions*/
//`Confirm General Setting;
int ErrGenSet(){
    string message;
    clear();
    message="From: MemoPad\n/"+
        MPCATF+" -> /"+MPCATT+"\n"+
        "To: SuperMemo\n/"+
        SMDB+" /"+SMCAT;
    return(
        alertc("Confirm",
            message, "OK:Cancel", 2));
}

//`Confirm MPCATs Validity;
int ValidMPCATs(){
    int i;
    string message;
    MPCatIDf=-1; MPCatIDt=-1;
    dbopen(MPDB);
    for(i=0; i<MaxMPDBCats; i++){
        if (dbcatname(i)==MPCATF)
            MPCatIDf=i;
        if (dbcatname(i)==MPCATT)
            MPCatIDt=i;
    }
    dbclose();
    if (MPCatIDf==-1 || MPCatIDt==-1){
        message="Invalid MemoPad "+
            "Category.\n"+
            "Conversion FAILED.";
        alertc("Error", message, "OK", 3);
        return(0);
    }
    return(1);
}

//`Confirm SMDB Validity;
int ValidSMDB(){
    string message;
    if (!dbopen(SmDB)){
        message="Invalid SuperMemo "+
            "DataBase.\n"+
            "Conversion FAILED.";
        alertc("Error", message, "OK", 3);
        dbclose();
        return(0);
    }
    dbclose();
    return(1);
}

//`Confirm SMCAT Validity;
int ValidSMCAT(){
    string smCat, message;
    int i, smCatAmt;
    if (SMCAT=="Unfiled"){
        SMCatID=0xffff;
        SMCatRecID=0xf;
        return(1);
    }
    dbopen(SmDB);
    dbrec(0xa);
    smCatAmt=(dbsize()-0x20)/2;
    for(i=0; i<smCatAmt; i++){
        dbrec(0xa); //SMCatID Index;
        dbseek(0x20+i*2);
        dbreadx(&SMCatID, "i2");
        dbrec(0xb); //SMCatRecID Index;
        dbseek((0x20)+SMCatID*2);
        dbreadx(&SMCatRecID, "i2");
        dbrec(SMCatRecID);
        dbseek(0x2); //SMCat;
        dbreadx(&smCat, "sz");
        if (smCat==SMCAT){
            dbclose();
            return(1);
        }
    } //for(i=0;
    dbclose();
    message="Invalid SuperMemo "+
        "Category.\n"+
        "Conversion FAILD.";
    alertc("Error", message, "OK", 3);
    return(0);
}

//`Calculate Relative Amounts;
int CalcAmts(pointer mpCardAmt,
    pointer smCardAmt){
    int i;
    (*mpCardAmt)=0;
    dbopen(MPDB);
    for(i=0; i<dbnrecs(); i++){
        dbrec(i);
        if (dbgetcat()==MPCatIDf)
            (*mpCardAmt)++;
    }
    dbclose();
    dbopen(SmDB);
    (*smCardAmt)=
        MaxSMDBRecs-dbnrecs();
    dbclose();
    return(1);
}

//`Calculate FieldLocs and FieldLens;
int CalcFields(){
    char c;
    int i, recLoc, exc;
    FieldLoc[0]=0;
    recLoc=0; c=dbread('c');
    while(c!=0xa && c!=0x0 &&
        recLoc<MaxFieldLen){
        recLoc++; dbseek(recLoc); 
        c=dbread('c');
    }
    FieldLen[0]=recLoc;
    if (c==0xa) FieldLoc[1]=recLoc+1;
    else FieldLoc[1]=recLoc;
    for(i=2; i<=6; i++){
        if (MaxFieldLen<=
            dbsize()-1-FieldLoc[i-1]){
                recLoc=
                    FieldLoc[i-1]+MaxFieldLen;
                dbseek(recLoc); c=dbread('c');
                while(c!=0x20 && c!=0xa &&
                    recLoc>FieldLoc[i-1]){
                    recLoc--; dbseek(recLoc);
                    c=dbread('c');
                }
                if (recLoc==FieldLoc[i-1]){
                    FieldLen[i-1]=MaxFieldLen;
                    exc=0;
                }else{
                    FieldLen[i-1]=
                        recLoc-FieldLoc[i-1];
                    exc=1;
                }
        }else{ //if (MaxFieldLen<=;
            FieldLen[i-1]=
                dbsize()-1-FieldLoc[i-1];
            exc=0;
        }
        if (i<6) FieldLoc[i]=
            FieldLoc[i-1]+FieldLen[i-1]+exc;
    } //for(i=2;
    return(1);
}

//`Get Records From MemoPad;
int GetRecs(){
    int i;
    CalcFields();
    for(i=0; i<6; i++){
        Field[i]="";
        if (FieldLen[i]>0){
            dbseek(FieldLoc[i]);
            dbreadx(
                &Field[i], "s"+FieldLen[i]);
        }
    }
    return(1);
}

//`Check SMDB Volume;
int SMDBFull(){
    string message;
    if (dbnrecs()>=MaxSMDBRecs){
        message="SuperMemo DateDase"
            +" FULL.\n"
            +"Conversion Interrupted.";
        alertc("Notice",
            message, "OK", 2);
        return(1);
        }
    return(0);
}

//`Get Create Date;
int CreatDate(){
    int year, month, day;
    year=date(0)/10000;
    month=(date(0)%10000)/100;
    day=date(0)%100;
    return((year-1904)*2*0x100+
        month*2*0x10+day);
}

//`Put Reconds to SuperMemo;
int PutRecs(){
    int i, smCardRecID;
    smCardRecID=dbnrecs();
    dbrec(smCardRecID);
    dbwrite(0x00020000+SMCatID);
    dbwrite(0x00000000+CreatDate());
    dbwrite(0x00000000);
    dbwrite(0x00000000);
    dbwrite(0x00000000);
    for(i=0; i<6; i++)
        dbwrite(Field[i]);
    dbrec(SMCatRecID);
    dbseek(dbsize());
    dbwritex(&smCardRecID, "i2");
    return(1);
}

//`Display Status;
int DispStat(int cvtdCardAmt,
    int mpCardAmt, int smCardAmt){
    clear();
    puts("Record:\t\t"+Field[0]+"\n");
    puts("Converted:\t"+
        cvtdCardAmt+"\n");
    puts("Remaining:\t"+
       (mpCardAmt-cvtdCardAmt)+"\n");
    puts("Volume:\t\t"+
       (smCardAmt-cvtdCardAmt)+"\n");
    resetaot(); //ResetAutoOffTimer;
    return(1);
}

//`Display Result;
DispRslt(int cvtdCardAmt,
    int mpCardAmt, int smCardAmt){
    string message;
    sleep(500); //Wait for 0.5 sec;
    clear();
    message="Converted:\t"+
        cvtdCardAmt+
        "\nRemaining:\t"+
        (mpCardAmt-cvtdCardAmt)+
        "\nVolume:\t"+
        (smCardAmt-cvtdCardAmt);
    alertc("Result", message, "OK", 0);
    return(1);
}

//`Main Function;
main(){
    int mpCardAmt, smCardAmt,
           cvtdCardAmt, mpCardRecID;
    int i;
    
    //Preparations;
    hookhard(1);
    hooksilk(1);
    SmDB="sm_"+SMDB;
    
    //Confirmations;
    if (ErrGenSet()) return(0);
    if (!ValidMPCATs()) return(0);
    if (!ValidSMDB()) return(0);
    if (!ValidSMCAT()) return(0);
    
    //Converte Records;
    CalcAmts(&mpCardAmt,
        &smCardAmt);
    cvtdCardAmt=0;
    mpCardRecID=0;
    for(i=0; i<mpCardAmt; i++){
    
        //Get Records from MemoPad;
        dbopen(MPDB);
        dbrec(mpCardRecID++);
        while(dbgetcat()!=MPCatIDf)
            dbrec(mpCardRecID++);
        GetRecs();
        dbsetcat(MPCatIDt);
        dbclose();
        
        //Put Records to SuperMemo;
        dbopen(SmDB);
        if (SMDBFull()){
            dbclose();
            break;
        }
        PutRecs();
        dbclose();
        
        //Display Status;.
        DispStat(++cvtdCardAmt,
            mpCardAmt, smCardAmt);
        
    }// if(i=0;
    
    //Display Result;
    DispRslt(cvtdCardAmt,
        mpCardAmt, smCardAmt);
    
}